@@ -25,11 +25,11 @@ from sqlalchemy.orm import backref
 from sqlalchemy.orm.collections import attribute_mapped_collection
 from sqlalchemy.orm import exc
 
-from neutron.db import api as db_api
-from neutron.db import common_db_mixin
 from neutron.db import models_v2
+from neutron_lib.db import api as db_api
 from neutron_lib.db import constants as db_const
 from neutron_lib.db import model_base
+from neutron_lib.db import model_query
 from neutron_lib.db import utils as db_utils
 
 from networking_sfc.db import flowclassifier_db as fc_db
@@ -218,7 +218,6 @@ class ServiceGraph(model_base.BASEV2, model_base.HasId, model_base.HasProject):
 
 class SfcDbPlugin(
     ext_sfc.SfcPluginBase,
-    common_db_mixin.CommonDbMixin,
     ext_sg.ServiceGraphPluginBase
 ):
     """Mixin class to add port chain to db_plugin_base_v2."""
@@ -243,111 +242,111 @@ class SfcDbPlugin(
             },
             'chain_id': port_chain['chain_id'],
         }
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
+    @db_api.CONTEXT_READER
     def _validate_port_pair_groups(self, context, pg_ids, pc_id=None):
-        with db_api.context_manager.reader.using(context):
-            prev_pg_tap_enabled = False
-            for pg_id in pg_ids:
-                pg = self._get_port_pair_group(context, pg_id)
-                curr_pg_tap_enabled = pg['tap_enabled']
-                if prev_pg_tap_enabled and curr_pg_tap_enabled:
-                    raise ext_tap.ConsecutiveTapPPGNotSupported()
-                else:
-                    prev_pg_tap_enabled = curr_pg_tap_enabled
-            query = self._model_query(context, PortChain)
-            for port_chain_db in query.all():
-                if port_chain_db['id'] == pc_id:
-                    continue
-                pc_pg_ids = [
-                    assoc['portpairgroup_id']
-                    for assoc in port_chain_db.chain_group_associations
-                ]
-                if pc_pg_ids and pg_ids and pc_pg_ids == pg_ids:
-                    raise ext_sfc.InvalidPortPairGroups(
-                        port_pair_groups=pg_ids, port_chain=port_chain_db.id)
+        prev_pg_tap_enabled = False
+        for pg_id in pg_ids:
+            pg = self._get_port_pair_group(context, pg_id)
+            curr_pg_tap_enabled = pg['tap_enabled']
+            if prev_pg_tap_enabled and curr_pg_tap_enabled:
+                raise ext_tap.ConsecutiveTapPPGNotSupported()
+            prev_pg_tap_enabled = curr_pg_tap_enabled
+        query = model_query.query_with_hooks(context, PortChain)
+        for port_chain_db in query.all():
+            if port_chain_db['id'] == pc_id:
+                continue
+            pc_pg_ids = [
+                assoc['portpairgroup_id']
+                for assoc in port_chain_db.chain_group_associations
+            ]
+            if pc_pg_ids and pg_ids and pc_pg_ids == pg_ids:
+                raise ext_sfc.InvalidPortPairGroups(
+                    port_pair_groups=pg_ids, port_chain=port_chain_db.id)
 
+    @db_api.CONTEXT_READER
     def _validate_correlation_consistency(self, context, ppg_ids, pc_corr):
         # format like in ServiceFunctionParam.value to aid comparison later:
         pc_corr = jsonutils.dumps(pc_corr)
-        with db_api.context_manager.reader.using(context):
-            for ppg_id in ppg_ids:
-                ppg = self._get_port_pair_group(context, ppg_id)
-                for pp in ppg['port_pairs']:
-                    pp_corr = pp['service_function_parameters']['correlation']
-                    if pp_corr.value != 'null' and pp_corr.value != pc_corr:
-                        raise ext_sfc.PortChainInconsistentCorrelations(
-                            ppg=ppg_id)
+        for ppg_id in ppg_ids:
+            ppg = self._get_port_pair_group(context, ppg_id)
+            for pp in ppg['port_pairs']:
+                pp_corr = pp['service_function_parameters']['correlation']
+                if pp_corr.value != 'null' and pp_corr.value != pc_corr:
+                    raise ext_sfc.PortChainInconsistentCorrelations(
+                        ppg=ppg_id)
 
+    @db_api.CONTEXT_READER
     def _validate_flow_classifiers(self, context, fc_ids, pc_id=None):
-        with db_api.context_manager.reader.using(context):
-            fcs = [
-                self._get_flow_classifier(context, fc_id)
-                for fc_id in fc_ids
+        fcs = [
+            self._get_flow_classifier(context, fc_id)
+            for fc_id in fc_ids
+        ]
+        for fc in fcs:
+            fc_assoc = fc.chain_classifier_association
+            if fc_assoc and fc_assoc['portchain_id'] != pc_id:
+                raise ext_fc.FlowClassifierInUse(id=fc.id)
+
+        query = model_query.query_with_hooks(context, PortChain)
+        for port_chain_db in query.all():
+            if port_chain_db['id'] == pc_id:
+                continue
+            pc_fc_ids = [
+                assoc['flowclassifier_id']
+                for assoc in port_chain_db.chain_classifier_associations
             ]
-            for fc in fcs:
-                fc_assoc = fc.chain_classifier_association
-                if fc_assoc and fc_assoc['portchain_id'] != pc_id:
-                    raise ext_fc.FlowClassifierInUse(id=fc.id)
-
-            query = self._model_query(context, PortChain)
-            for port_chain_db in query.all():
-                if port_chain_db['id'] == pc_id:
-                    continue
-                pc_fc_ids = [
-                    assoc['flowclassifier_id']
-                    for assoc in port_chain_db.chain_classifier_associations
-                ]
-                pc_fcs = [
-                    self._get_flow_classifier(context, pc_fc_id)
-                    for pc_fc_id in pc_fc_ids
-                ]
-                for pc_fc in pc_fcs:
-                    for fc in fcs:
-                        fc_cls = fc_db.FlowClassifierDbPlugin
-                        if fc_cls.flowclassifier_basic_conflict(
-                            pc_fc, fc
-                        ):
-                            raise ext_sfc.PortChainFlowClassifierInConflict(
-                                fc_id=fc['id'], pc_id=port_chain_db['id'],
-                                pc_fc_id=pc_fc['id']
-                            )
+            pc_fcs = [
+                self._get_flow_classifier(context, pc_fc_id)
+                for pc_fc_id in pc_fc_ids
+            ]
+            for pc_fc in pc_fcs:
+                for fc in fcs:
+                    fc_cls = fc_db.FlowClassifierDbPlugin
+                    if fc_cls.flowclassifier_basic_conflict(
+                        pc_fc, fc
+                    ):
+                        raise ext_sfc.PortChainFlowClassifierInConflict(
+                            fc_id=fc['id'], pc_id=port_chain_db['id'],
+                            pc_fc_id=pc_fc['id']
+                        )
 
+    @db_api.CONTEXT_READER
     def _setup_chain_group_associations(
         self, context, port_chain, pg_ids
     ):
-        with db_api.context_manager.reader.using(context):
-            chain_group_associations = []
-            for pg_id in pg_ids:
-                query = self._model_query(context, ChainGroupAssoc)
-                chain_group_association = query.filter_by(
-                    portchain_id=port_chain.id, portpairgroup_id=pg_id
-                ).first()
-                if not chain_group_association:
-                    chain_group_association = ChainGroupAssoc(
-                        portpairgroup_id=pg_id
-                    )
-                chain_group_associations.append(chain_group_association)
-            port_chain.chain_group_associations = chain_group_associations
+        chain_group_associations = []
+        for pg_id in pg_ids:
+            query = model_query.query_with_hooks(context, ChainGroupAssoc)
+            chain_group_association = query.filter_by(
+                portchain_id=port_chain.id, portpairgroup_id=pg_id
+            ).first()
+            if not chain_group_association:
+                chain_group_association = ChainGroupAssoc(
+                    portpairgroup_id=pg_id
+                )
+            chain_group_associations.append(chain_group_association)
+        port_chain.chain_group_associations = chain_group_associations
 
+    @db_api.CONTEXT_READER
     def _setup_chain_classifier_associations(
         self, context, port_chain, fc_ids
     ):
-        with db_api.context_manager.reader.using(context):
-            chain_classifier_associations = []
-            for fc_id in fc_ids:
-                query = self._model_query(context, ChainClassifierAssoc)
-                chain_classifier_association = query.filter_by(
-                    portchain_id=port_chain.id, flowclassifier_id=fc_id
-                ).first()
-                if not chain_classifier_association:
-                    chain_classifier_association = ChainClassifierAssoc(
-                        flowclassifier_id=fc_id
-                    )
-                chain_classifier_associations.append(
-                    chain_classifier_association)
-            port_chain.chain_classifier_associations = (
-                chain_classifier_associations)
+        chain_classifier_associations = []
+        for fc_id in fc_ids:
+            query = model_query.query_with_hooks(
+                context, ChainClassifierAssoc)
+            chain_classifier_association = query.filter_by(
+                portchain_id=port_chain.id, flowclassifier_id=fc_id
+            ).first()
+            if not chain_classifier_association:
+                chain_classifier_association = ChainClassifierAssoc(
+                    flowclassifier_id=fc_id
+                )
+            chain_classifier_associations.append(
+                chain_classifier_association)
+        port_chain.chain_classifier_associations = (
+            chain_classifier_associations)
 
     @log_helpers.log_method_call
     def create_port_chain(self, context, port_chain):
@@ -355,7 +354,7 @@ class SfcDbPlugin(
         pc = port_chain['port_chain']
         project_id = pc['project_id']
         chain_id = pc['chain_id']
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             chain_parameters = {
                 key: ChainParameter(keyword=key, value=jsonutils.dumps(val))
                 for key, val in pc['chain_parameters'].items()}
@@ -400,24 +399,27 @@ class SfcDbPlugin(
             return self._make_port_chain_dict(port_chain_db)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_port_chains(self, context, filters=None, fields=None,
                         sorts=None, limit=None,
                         marker=None, page_reverse=False, default_sg=False):
         marker_obj = db_utils.get_marker_obj(self, context, 'port_chain',
                                              limit, marker)
-        return self._get_collection(context,
-                                    PortChain,
-                                    self._make_port_chain_dict,
-                                    filters=filters, fields=fields,
-                                    sorts=sorts,
-                                    limit=limit, marker_obj=marker_obj,
-                                    page_reverse=page_reverse)
+        return model_query.get_collection(
+            context, PortChain,
+            self._make_port_chain_dict,
+            filters=filters, fields=fields,
+            sorts=sorts,
+            limit=limit, marker_obj=marker_obj,
+            page_reverse=page_reverse)
 
+    @db_api.CONTEXT_READER
     def get_port_chains_count(self, context, filters=None):
-        return self._get_collection_count(context, PortChain,
-                                          filters=filters)
+        return model_query.get_collection_count(
+            context, PortChain, filters=filters)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_port_chain(self, context, id, fields=None):
         portchain = self._get_port_chain(context, id)
         return self._make_port_chain_dict(portchain, fields)
@@ -425,7 +427,7 @@ class SfcDbPlugin(
     @log_helpers.log_method_call
     def _get_port_chain(self, context, id):
         try:
-            return self._get_by_id(context, PortChain, id)
+            return model_query.get_by_id(context, PortChain, id)
         except exc.NoResultFound:
             raise ext_sfc.PortChainNotFound(id=id)
 
@@ -435,7 +437,7 @@ class SfcDbPlugin(
         if self._any_port_chains_in_a_graph(context, {id}):
             raise ext_sg.ServiceGraphPortChainInUse(id=id)
         try:
-            with db_api.context_manager.writer.using(context):
+            with db_api.CONTEXT_WRITER.using(context):
                 pc = self._get_port_chain(context, id)
                 context.session.delete(pc)
         except ext_sfc.PortChainNotFound:
@@ -449,7 +451,7 @@ class SfcDbPlugin(
             if 'flow_classifiers' in pc or 'port_pair_groups' in pc:
                 raise ext_sg.ServiceGraphPortChainInUse(id=id)
 
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             pc_db = self._get_port_chain(context, id)
             for k, v in pc.items():
                 if k == 'flow_classifiers':
@@ -481,7 +483,7 @@ class SfcDbPlugin(
             }
         }
 
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
     def _validate_port_pair_ingress_egress(self, ingress, egress):
         if 'device_id' not in ingress or not ingress['device_id']:
@@ -502,8 +504,8 @@ class SfcDbPlugin(
         """Create a port pair."""
         pp = port_pair['port_pair']
         project_id = pp['project_id']
-        with db_api.context_manager.writer.using(context):
-            query = self._model_query(context, PortPair)
+        with db_api.CONTEXT_WRITER.using(context):
+            query = model_query.query_with_hooks(context, PortPair)
             pp_in_use = query.filter_by(
                 ingress=pp['ingress'], egress=pp['egress']
             ).first()
@@ -536,44 +538,47 @@ class SfcDbPlugin(
             return self._make_port_pair_dict(port_pair_db)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_port_pairs(self, context, filters=None, fields=None,
                        sorts=None, limit=None, marker=None,
                        page_reverse=False):
         marker_obj = db_utils.get_marker_obj(self, context, 'port_pair',
                                              limit, marker)
-        return self._get_collection(context,
-                                    PortPair,
-                                    self._make_port_pair_dict,
-                                    filters=filters, fields=fields,
-                                    sorts=sorts,
-                                    limit=limit, marker_obj=marker_obj,
-                                    page_reverse=page_reverse)
+        return model_query.get_collection(
+            context, PortPair,
+            self._make_port_pair_dict,
+            filters=filters, fields=fields,
+            sorts=sorts,
+            limit=limit, marker_obj=marker_obj,
+            page_reverse=page_reverse)
 
+    @db_api.CONTEXT_READER
     def get_port_pairs_count(self, context, filters=None):
-        return self._get_collection_count(context, PortPair,
-                                          filters=filters)
+        return model_query.get_collection_count(
+            context, PortPair, filters=filters)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_port_pair(self, context, id, fields=None):
         port_pair = self._get_port_pair(context, id)
         return self._make_port_pair_dict(port_pair, fields)
 
     def _get_port_pair(self, context, id):
         try:
-            return self._get_by_id(context, PortPair, id)
+            return model_query.get_by_id(context, PortPair, id)
         except exc.NoResultFound:
             raise ext_sfc.PortPairNotFound(id=id)
 
     def _get_port(self, context, id):
         try:
-            return self._get_by_id(context, models_v2.Port, id)
+            return model_query.get_by_id(context, models_v2.Port, id)
         except exc.NoResultFound:
             raise ext_sfc.PortPairPortNotFound(id=id)
 
     @log_helpers.log_method_call
     def update_port_pair(self, context, id, port_pair):
         new_pp = port_pair['port_pair']
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             old_pp = self._get_port_pair(context, id)
             old_pp.update(new_pp)
             return self._make_port_pair_dict(old_pp)
@@ -581,7 +586,7 @@ class SfcDbPlugin(
     @log_helpers.log_method_call
     def delete_port_pair(self, context, id):
         try:
-            with db_api.context_manager.writer.using(context):
+            with db_api.CONTEXT_WRITER.using(context):
                 pp = self._get_port_pair(context, id)
                 if pp.portpairgroup_id:
                     raise ext_sfc.PortPairInUse(id=id)
@@ -605,7 +610,7 @@ class SfcDbPlugin(
             'group_id': port_pair_group.get('group_id') or 0
         }
 
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
     def _validate_pps_in_ppg(self, portpairs_list, id=None):
         first_check = True
@@ -632,7 +637,7 @@ class SfcDbPlugin(
         pg = port_pair_group['port_pair_group']
         project_id = pg['project_id']
 
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             portpairs_list = [self._get_port_pair(context, pp_id)
                               for pp_id in pg['port_pairs']]
             self._validate_pps_in_ppg(portpairs_list)
@@ -671,37 +676,41 @@ class SfcDbPlugin(
             return self._make_port_pair_group_dict(port_pair_group_db)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_port_pair_groups(self, context, filters=None, fields=None,
                              sorts=None, limit=None, marker=None,
                              page_reverse=False):
         marker_obj = db_utils.get_marker_obj(self, context, 'port_pair_group',
                                              limit, marker)
-        return self._get_collection(context,
-                                    PortPairGroup,
-                                    self._make_port_pair_group_dict,
-                                    filters=filters, fields=fields,
-                                    sorts=sorts,
-                                    limit=limit, marker_obj=marker_obj,
-                                    page_reverse=page_reverse)
+        return model_query.get_collection(
+            context, PortPairGroup,
+            self._make_port_pair_group_dict,
+            filters=filters, fields=fields,
+            sorts=sorts,
+            limit=limit, marker_obj=marker_obj,
+            page_reverse=page_reverse)
 
+    @db_api.CONTEXT_READER
     def get_port_pair_groups_count(self, context, filters=None):
-        return self._get_collection_count(context, PortPairGroup,
-                                          filters=filters)
+        return model_query.get_collection_count(
+            context, PortPairGroup, filters=filters)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_port_pair_group(self, context, id, fields=None):
         port_pair_group = self._get_port_pair_group(context, id)
         return self._make_port_pair_group_dict(port_pair_group, fields)
 
     def _get_port_pair_group(self, context, id):
         try:
-            return self._get_by_id(context, PortPairGroup, id)
+            return model_query.get_by_id(context, PortPairGroup, id)
         except exc.NoResultFound:
             raise ext_sfc.PortPairGroupNotFound(id=id)
 
+    @db_api.CONTEXT_READER
     def _get_flow_classifier(self, context, id):
         try:
-            return self._get_by_id(context, fc_db.FlowClassifier, id)
+            return model_query.get_by_id(context, fc_db.FlowClassifier, id)
         except exc.NoResultFound:
             raise ext_fc.FlowClassifierNotFound(id=id)
 
@@ -709,7 +718,7 @@ class SfcDbPlugin(
     def update_port_pair_group(self, context, id, port_pair_group):
         new_pg = port_pair_group['port_pair_group']
 
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             portpairs_list = [self._get_port_pair(context, pp_id)
                               for pp_id in new_pg.get('port_pairs', [])]
             self._validate_pps_in_ppg(portpairs_list, id)
@@ -731,7 +740,7 @@ class SfcDbPlugin(
     @log_helpers.log_method_call
     def delete_port_pair_group(self, context, id):
         try:
-            with db_api.context_manager.writer.using(context):
+            with db_api.CONTEXT_WRITER.using(context):
                 pg = self._get_port_pair_group(context, id)
                 if pg.chain_group_associations:
                     raise ext_sfc.PortPairGroupInUse(id=id)
@@ -757,7 +766,7 @@ class SfcDbPlugin(
             'port_chains': self._graph_assocs_to_pc_dict(
                 graph_db['graph_chain_associations'])
         }
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
     def _make_graph_chain_assoc_dict(self, assoc_db, fields=None):
         res = {
@@ -765,7 +774,7 @@ class SfcDbPlugin(
             'src_chain': assoc_db['src_chain'],
             'dst_chain': assoc_db['dst_chain']
         }
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
     def _is_there_a_loop(self, parenthood, current_chain, traversed):
         if current_chain not in parenthood:
@@ -783,8 +792,8 @@ class SfcDbPlugin(
                                     port_chains=set(), graph_id=None):
         if not port_chains:
             return False
-        with db_api.context_manager.reader.using(context):
-            query = self._model_query(context, ServiceGraph)
+        with db_api.CONTEXT_READER.using(context):
+            query = model_query.query_with_hooks(context, ServiceGraph)
             for graph_db in query.all():
                 if graph_db['id'] == graph_id:
                     continue
@@ -803,94 +812,94 @@ class SfcDbPlugin(
 
     def _validate_port_chains_for_graph(self, context,
                                         port_chains, graph_id=None):
-            # create a list of all port-chains that will be associated
-            all_port_chains = set()
-            for src_chain in port_chains:
-                all_port_chains.add(src_chain)
-                for dst_chain in port_chains[src_chain]:
-                    all_port_chains.add(dst_chain)
-            # check if any of the port-chains are already in a graph
-            if self._any_port_chains_in_a_graph(
-                    context, all_port_chains, graph_id):
-                raise ext_sg.ServiceGraphInvalidPortChains(
-                    port_chains=port_chains)
+        # create a list of all port-chains that will be associated
+        all_port_chains = set()
+        for src_chain in port_chains:
+            all_port_chains.add(src_chain)
+            for dst_chain in port_chains[src_chain]:
+                all_port_chains.add(dst_chain)
+        # check if any of the port-chains are already in a graph
+        if self._any_port_chains_in_a_graph(
+                context, all_port_chains, graph_id):
+            raise ext_sg.ServiceGraphInvalidPortChains(
+                port_chains=port_chains)
 
-            # dict whose keys are PCs and values are lists of dependency-PCs
-            # (PCs incoming to the point where the key is a outgoing)
-            parenthood = {}
-            encapsulation = None
-            fc_cls = fc_db.FlowClassifierDbPlugin
-            for src_chain in port_chains:
-                src_pc = self._get_port_chain(context, src_chain)
-                curr_corr = src_pc.chain_parameters['correlation']['value']
-                # guarantee that branching PPG supports correlation
-                assocs = src_pc.chain_group_associations
-                src_ppg = max(assocs, key=(lambda ppg: ppg.position))
-                ppg_id = src_ppg['portpairgroup_id']
+        # dict whose keys are PCs and values are lists of dependency-PCs
+        # (PCs incoming to the point where the key is a outgoing)
+        parenthood = {}
+        encapsulation = None
+        fc_cls = fc_db.FlowClassifierDbPlugin
+        for src_chain in port_chains:
+            src_pc = self._get_port_chain(context, src_chain)
+            curr_corr = src_pc.chain_parameters['correlation']['value']
+            # guarantee that branching PPG supports correlation
+            assocs = src_pc.chain_group_associations
+            src_ppg = max(assocs, key=(lambda ppg: ppg.position))
+            ppg_id = src_ppg['portpairgroup_id']
+            ppg = self._get_port_pair_group(context, ppg_id)
+            for pp in ppg.port_pairs:
+                sfparams = pp['service_function_parameters']
+                if sfparams['correlation']['value'] != curr_corr:
+                    raise ext_sg.ServiceGraphImpossibleBranching()
+
+            # verify encapsulation consistency across all PCs (part 1)
+            if not encapsulation:
+                encapsulation = curr_corr
+            elif encapsulation != curr_corr:
+                raise ext_sg.ServiceGraphInconsistentEncapsulation()
+            # list of all port chains at this branching point:
+            branching_point = []
+            # list of every flow classifier at this branching point:
+            fcs_for_src_chain = []
+            for dst_chain in port_chains[src_chain]:
+                # check if the current destination PC was already added
+                if dst_chain in branching_point:
+                    raise ext_sg.ServiceGraphPortChainInConflict(
+                        pc_id=dst_chain)
+                branching_point.append(dst_chain)
+                dst_pc = self._get_port_chain(context, dst_chain)
+                curr_corr = dst_pc.chain_parameters['correlation']['value']
+                # guarantee that destination PPG supports correlation
+                assocs = dst_pc.chain_group_associations
+                dst_ppg = min(assocs, key=(lambda ppg: ppg.position))
+                ppg_id = dst_ppg['portpairgroup_id']
                 ppg = self._get_port_pair_group(context, ppg_id)
                 for pp in ppg.port_pairs:
                     sfparams = pp['service_function_parameters']
                     if sfparams['correlation']['value'] != curr_corr:
                         raise ext_sg.ServiceGraphImpossibleBranching()
-
-                # verify encapsulation consistency across all PCs (part 1)
-                if not encapsulation:
-                    encapsulation = curr_corr
-                elif encapsulation != curr_corr:
+                # verify encapsulation consistency across all PCs (part 2)
+                if encapsulation != curr_corr:
                     raise ext_sg.ServiceGraphInconsistentEncapsulation()
-                # list of all port chains at this branching point:
-                branching_point = []
-                # list of every flow classifier at this branching point:
-                fcs_for_src_chain = []
-                for dst_chain in port_chains[src_chain]:
-                    # check if the current destination PC was already added
-                    if dst_chain in branching_point:
-                        raise ext_sg.ServiceGraphPortChainInConflict(
-                            pc_id=dst_chain)
-                    branching_point.append(dst_chain)
-                    dst_pc = self._get_port_chain(context, dst_chain)
-                    curr_corr = dst_pc.chain_parameters['correlation']['value']
-                    # guarantee that destination PPG supports correlation
-                    assocs = dst_pc.chain_group_associations
-                    dst_ppg = min(assocs, key=(lambda ppg: ppg.position))
-                    ppg_id = dst_ppg['portpairgroup_id']
-                    ppg = self._get_port_pair_group(context, ppg_id)
-                    for pp in ppg.port_pairs:
-                        sfparams = pp['service_function_parameters']
-                        if sfparams['correlation']['value'] != curr_corr:
-                            raise ext_sg.ServiceGraphImpossibleBranching()
-                    # verify encapsulation consistency across all PCs (part 2)
-                    if encapsulation != curr_corr:
-                        raise ext_sg.ServiceGraphInconsistentEncapsulation()
-                    dst_pc_dict = self._make_port_chain_dict(dst_pc)
-                    # acquire associated flow classifiers
-                    fcs = dst_pc_dict['flow_classifiers']
-                    for fc_id in fcs:
-                        fc = self._get_flow_classifier(context, fc_id)
-                        fcs_for_src_chain.append(fc)  # update list of every FC
-                    # update the parenthood dict
-                    if dst_chain in parenthood:
-                        parenthood[dst_chain].append(src_chain)
-                    else:
-                        parenthood[dst_chain] = [src_chain]
-                    # detect duplicate FCs, consequently branching ambiguity
-                    for i, fc1 in enumerate(fcs_for_src_chain):
-                        for fc2 in fcs_for_src_chain[i + 1:]:
-                            if(fc_cls.flowclassifier_basic_conflict(fc1, fc2)):
-                                raise ext_sg.\
-                                    ServiceGraphFlowClassifierInConflict(
-                                        fc1_id=fc1['id'], fc2_id=fc2['id'])
+                dst_pc_dict = self._make_port_chain_dict(dst_pc)
+                # acquire associated flow classifiers
+                fcs = dst_pc_dict['flow_classifiers']
+                for fc_id in fcs:
+                    fc = self._get_flow_classifier(context, fc_id)
+                    fcs_for_src_chain.append(fc)  # update list of every FC
+                # update the parenthood dict
+                if dst_chain in parenthood:
+                    parenthood[dst_chain].append(src_chain)
+                else:
+                    parenthood[dst_chain] = [src_chain]
+                # detect duplicate FCs, consequently branching ambiguity
+                for i, fc1 in enumerate(fcs_for_src_chain):
+                    for fc2 in fcs_for_src_chain[i + 1:]:
+                        if(fc_cls.flowclassifier_basic_conflict(fc1, fc2)):
+                            raise ext_sg.\
+                                ServiceGraphFlowClassifierInConflict(
+                                    fc1_id=fc1['id'], fc2_id=fc2['id'])
 
-            # check for circular paths within the graph via parenthood dict:
-            for port_chain in parenthood:
-                if self._is_there_a_loop(parenthood, port_chain, []):
-                    raise ext_sg.ServiceGraphLoopDetected()
+        # check for circular paths within the graph via parenthood dict:
+        for port_chain in parenthood:
+            if self._is_there_a_loop(parenthood, port_chain, []):
+                raise ext_sg.ServiceGraphLoopDetected()
 
     def _setup_graph_chain_associations(self, context, graph_db, port_chains):
-        with db_api.context_manager.reader.using(context):
+        with db_api.CONTEXT_READER.using(context):
             graph_chain_associations = []
             for src_chain in port_chains:
-                query = self._model_query(context, GraphChainAssoc)
+                query = model_query.query_with_hooks(context, GraphChainAssoc)
                 for dst_chain in port_chains[src_chain]:
                     graph_chain_association = query.filter_by(
                         service_graph_id=graph_db.id,
@@ -904,18 +913,19 @@ class SfcDbPlugin(
                     graph_chain_associations.append(graph_chain_association)
             graph_db.graph_chain_associations = graph_chain_associations
 
-    def _get_branches(self, context, filters):
-        return self._get_collection(context,
-                                    GraphChainAssoc,
-                                    self._make_graph_chain_assoc_dict,
-                                    filters=filters)
+    @db_api.CONTEXT_READER
+    def get_branches(self, context, filters):
+        return model_query.get_collection(
+            context, GraphChainAssoc,
+            self._make_graph_chain_assoc_dict,
+            filters=filters)
 
     @log_helpers.log_method_call
     def create_service_graph(self, context, service_graph):
         """Create a Service Graph."""
         service_graph = service_graph['service_graph']
         project_id = service_graph['project_id']
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             port_chains = service_graph['port_chains']
             self._validate_port_chains_for_graph(context, port_chains)
             graph_db = ServiceGraph(id=uuidutils.generate_uuid(),
@@ -928,21 +938,23 @@ class SfcDbPlugin(
             return self._make_service_graph_dict(graph_db)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_service_graphs(self, context, filters=None,
                            fields=None, sorts=None, limit=None,
                            marker=None, page_reverse=False):
         """Get Service Graphs."""
         marker_obj = db_utils.get_marker_obj(self, context,
                                              'service_graph', limit, marker)
-        return self._get_collection(context,
-                                    ServiceGraph,
-                                    self._make_service_graph_dict,
-                                    filters=filters, fields=fields,
-                                    sorts=sorts,
-                                    limit=limit, marker_obj=marker_obj,
-                                    page_reverse=page_reverse)
+        return model_query.get_collection(
+            context, ServiceGraph,
+            self._make_service_graph_dict,
+            filters=filters, fields=fields,
+            sorts=sorts,
+            limit=limit, marker_obj=marker_obj,
+            page_reverse=page_reverse)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_service_graph(self, context, id, fields=None):
         """Get a Service Graph."""
         service_graph = self._get_service_graph(context, id)
@@ -951,7 +963,7 @@ class SfcDbPlugin(
     @log_helpers.log_method_call
     def _get_service_graph(self, context, id):
         try:
-            return self._get_by_id(context, ServiceGraph, id)
+            return model_query.get_by_id(context, ServiceGraph, id)
         except exc.NoResultFound:
             raise ext_sg.ServiceGraphNotFound(id=id)
 
@@ -959,7 +971,7 @@ class SfcDbPlugin(
     def update_service_graph(self, context, id, service_graph):
         """Update a Service Graph."""
         service_graph = service_graph['service_graph']
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             graph_db = self._get_service_graph(context, id)
             for k, v in service_graph.items():
                 if k == 'port_chains':
@@ -975,7 +987,7 @@ class SfcDbPlugin(
     def delete_service_graph(self, context, id):
         """Delete a Service Graph."""
         try:
-            with db_api.context_manager.writer.using(context):
+            with db_api.CONTEXT_WRITER.using(context):
                 graph = self._get_service_graph(context, id)
                 context.session.delete(graph)
         except ext_sfc.ServiceGraphNotFound:
